package pc.javier.actualizadoropendns.control.receptor;


import android.app.Activity;

import java.util.Date;

import pc.javier.actualizadoropendns.R;
import pc.javier.actualizadoropendns.adaptador.Pantalla;
import pc.javier.actualizadoropendns.adaptador.Preferencias;
import pc.javier.actualizadoropendns.bd.BD;
import pc.javier.actualizadoropendns.control.Aplicacion;
import utilidades.FechaHora;
import utilidades.InfoInternet;
import utilidades.MensajeRegistro;
import utilidades.ReceptorEventos;

/**
 * Javier 2019.
 *  Recibe comandos por parte del Servidor
 */

public class ReceptorComandosInternet extends ReceptorEventos{
    private Preferencias preferencias;
    private Activity activity;
    private Pantalla pantallaPrincipal;
    private BD bd;

    public ReceptorComandosInternet(Activity activity, String clave) {
        super(clave);
        this.activity = activity;
        preferencias = new Preferencias(activity);
        pantallaPrincipal = new Pantalla(Aplicacion.actividadPrincipal);
        bd = new BD(activity);
    }

    @Override
    public void procesar(String dato) {
        MensajeRegistro.msj("receptor COMANDOS - " +dato);

        // al dato lo "subdivide" en comando y parametro
        int indice = dato.indexOf(" ");
        String comando = dato;
        String parametro = "";

        if (indice > 0) {
            comando = dato.substring(0, indice);
            parametro = dato.substring(indice + 1);
        }

        ejecutar (comando, parametro);
    }





    private void ejecutar (String comando, String parametro) {

        int error = 0;
        switch (comando) {

            case "badauth":
                pantallaPrincipal.snack(R.string.claveIncorrecta);
                error = 1;
                break;
            case "nohost":
                pantallaPrincipal.snack(R.string.perfilIncorrecto);
                error = 1;
                break;
            case "good":
            case "!yours":
                pantallaPrincipal.snack(R.string.actualizacionCorrecta + " " + parametro);
                pantallaPrincipal.setTextView(R.id.ultimaActualizacion, "Ahora");
                preferencias.setultimaActualizacion(new Date().getTime());
                break;
            case "abuse":
                pantallaPrincipal.snack(R.string.bloqueado);
                error = 1;
                break;
        }

        InfoInternet infoInternet = new InfoInternet(activity);
        String informacion = infoInternet.getTipo() + ": " + infoInternet.getInfo();
        FechaHora fechaHora = new FechaHora();
        bd.abrir();
        bd.registroAgregar(fechaHora.obtenerHoraFechaNormal(), comando + " " + parametro, informacion, error);
        bd.cerrar();
    }




    // ------------------------------------------- COMANDOS ----






    // -----------------------------------------------------

    private int  entero (String numero) {
        try { return Integer.parseInt(numero); }
            catch (Exception e) { return 0; }
    }

    private boolean boleano (String valor) {
        try { return Boolean.parseBoolean(valor); }
        catch (Exception e) { return false; }
    }






    private void mensajeLog (String texto) { MensajeRegistro.msj("RECEPTOR COMANDOS", texto);}
}
